program qfpu
  implicit none

  integer(kind=8),external  :: timebase
  real(kind=8),parameter    :: clockspeed = 1600.d6

  integer,parameter:: nelt  = 800
  integer,parameter:: niter = 1000000

  integer :: i, iter
  integer(kind=8) :: flop

  real(kind=8),dimension(nelt) :: a,b,aok,bok
  real(kind=8) :: c,maxerr
!IBM* align(32,a,b)
  vector(real(8)) :: avec,bvec,cvec1,cvec2

  integer(kind=8) :: t0, t1
  real(kind=8) :: mflops,totaltime

  ! Premier noyau de calcul
  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
    do i = 1,nelt
      a(i) = a(i) + c*b(i)
      b(i) = a(i) - c*b(i)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 1 : ',mflops,'MFlops, time=',totaltime,'s'

  aok(:)=a(:)
  bok(:)=b(:)




  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
!IBM*INDEPENDENT
    do i = 1,nelt
      a(i) = a(i) + c*b(i)
      b(i) = a(i) - c*b(i)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 1 indep : ',mflops,'MFlops, time=',totaltime,'s'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-8) print *,'Max error=',maxerr





  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    cvec1 = vec_splats(b(1))
    cvec2 = vec_splats(-b(1))
!IBM*INDEPENDENT
    do i = 1,nelt,4
      avec = vec_ld(0,a(i))
      bvec = vec_ld(0,b(i))
      avec = vec_madd(cvec1,bvec,avec)
      bvec = vec_madd(cvec2,bvec,avec)
      call vec_st(avec,0,a(i))
      call vec_st(bvec,0,b(i))
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 1 intrins 1 : ',mflops,'MFlops, time=',totaltime,'s'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-8) print *,'Max error=',maxerr




  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    cvec1 = vec_splats(b(1))
!IBM*INDEPENDENT
    do i = 1,nelt,4
      avec = vec_ld(0,a(i))
      bvec = vec_ld(0,b(i))
      call vec_st(avec,0,b(i))
      avec = vec_madd(cvec1,bvec,avec)
      call vec_st(avec,0,a(i))
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 1 intrins 2 : ',mflops,'MFlops, time=',totaltime,'s (true MFlops:',mflops/2,')'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-8) print *,'Max error=',maxerr



  ! Deuxieme noyau de calcul
  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
    do i = 1,nelt,4
      a(i)   = a(i)   + c*b(i)
      b(i)   = a(i)   - c*b(i)
      a(i+1) = a(i+1) - c*b(i+1)
      b(i+1) = a(i+1) + c*b(i+1)
      a(i+2) = a(i+2) + c*b(i+2)
      b(i+2) = a(i+2) - c*b(i+2)
      a(i+3) = a(i+3) - c*b(i+3)
      b(i+3) = a(i+3) + c*b(i+3)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 2 : ',mflops,'MFlops, time=',totaltime,'s'

  aok(:)=a(:)
  bok(:)=b(:)





  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
!IBM*INDEPENDENT
    do i = 1,nelt,4
      a(i)   = a(i)   + c*b(i)
      b(i)   = a(i)   - c*b(i)
      a(i+1) = a(i+1) - c*b(i+1)
      b(i+1) = a(i+1) + c*b(i+1)
      a(i+2) = a(i+2) + c*b(i+2)
      b(i+2) = a(i+2) - c*b(i+2)
      a(i+3) = a(i+3) - c*b(i+3)
      b(i+3) = a(i+3) + c*b(i+3)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 2 indep : ',mflops,'MFlops, time=',totaltime,'s'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-8) print *,'Max error=',maxerr




  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    cvec1=vec_insert( b(1),cvec1,1)
    cvec1=vec_insert(-b(1),cvec1,2)
    cvec1=vec_insert( b(1),cvec1,3)
    cvec1=vec_insert(-b(1),cvec1,4)
    cvec2=vec_insert(-b(1),cvec2,1)
    cvec2=vec_insert( b(1),cvec2,2)
    cvec2=vec_insert(-b(1),cvec2,3)
    cvec2=vec_insert( b(1),cvec2,4)
!IBM*INDEPENDENT
    do i = 1,nelt,4
      avec = vec_ld(0,a(i))
      bvec = vec_ld(0,b(i))
      avec = vec_madd(cvec1,bvec,avec)
      bvec = vec_madd(cvec2,bvec,avec)
      call vec_st(avec,0,a(i))
      call vec_st(bvec,0,b(i))
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 2 intrins 1 : ',mflops,'MFlops, time=',totaltime,'s'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-7) print *,'Max error=',maxerr





  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    cvec1=vec_insert( b(1),cvec1,1)
    cvec1=vec_insert(-b(1),cvec1,2)
    cvec1=vec_insert( b(1),cvec1,3)
    cvec1=vec_insert(-b(1),cvec1,4)
!IBM*INDEPENDENT
    do i = 1,nelt,4
      avec = vec_ld(0,a(i))
      bvec = vec_ld(0,b(i))
      call vec_st(avec,0,b(i))
      avec = vec_madd(cvec1,bvec,avec)
      call vec_st(avec,0,a(i))
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 2 intrins 2 : ',mflops,'MFlops, time=',totaltime,'s (true MFlops:',mflops/2,')'

  maxerr = -1.d0
  do i=1,nelt
    maxerr=max(maxerr,abs(bok(i)-b(i)))
    maxerr=max(maxerr,abs(aok(i)-a(i)))
  end do
  if (maxerr>1e-7) print *,'Max error=',maxerr


end program 
