program qfpu
  implicit none

  integer(kind=8),external  :: timebase
  real(kind=8),parameter    :: clockspeed = 1600.d6

  integer,parameter:: nelt  = 800
  integer,parameter:: niter = 1000000

  integer :: i, iter
  integer(kind=8) :: flop

  real(kind=8),dimension(nelt) :: a,b
  real(kind=8) :: c
  real(kind=8),dimension(2) :: cvec

  integer(kind=8) :: t0, t1
  real(kind=8) :: mflops,totaltime

  ! Premier noyau de calcul
  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
    do i = 1,nelt
      a(i) = a(i) + c*b(i)
      b(i) = a(i) - c*b(i)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 1 : ',mflops,'MFlops, time=',totaltime,'s'


  ! Deuxieme noyau de calcul
  do i=1,nelt
    a(i)=0.0000001*(1.d0+sin(real(i,kind=8)))
    b(i)=-0.0000000123*(1.d0+cos(real(i+2,kind=8)))
  end do

  t0 = timebase()

  c = 1.d0
  do iter = 1,niter
    c = b(1)
    do i = 1,nelt,4
      a(i)   = a(i)   + c*b(i)
      b(i)   = a(i)   - c*b(i)
      a(i+1) = a(i+1) - c*b(i+1)
      b(i+1) = a(i+1) + c*b(i+1)
      a(i+2) = a(i+2) + c*b(i+2)
      b(i+2) = a(i+2) - c*b(i+2)
      a(i+3) = a(i+3) - c*b(i+3)
      b(i+3) = a(i+3) + c*b(i+3)
    end do
  end do

  t1 = timebase()

  flop = 4_8*nelt*niter
  totaltime = (t1 - t0)/clockspeed
  mflops=dble(flop)/(totaltime*1.d6)

  print *,'Performance noyau 2 : ',mflops,'MFlops, time=',totaltime,'s'

end program 
