!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! erreur.f90  --- Calcul de l'erreur dans un sous-domaine.
!!                 Maximum de la valeur absolue de la difference de u a
!!                 l'iteration n et de u a l'iteration n+1.
!! 
!! Auteur          : Isabelle DUPAYS (CNRS/IDRIS - France) 
!!                   <Isabelle.Dupays@idris.fr>
!! Cree le         : Fri Nov 15 10:32:45 1996
!! Modifie par     : Philippe WAUTELET (CNRS/IDRIS - France)
!!                   <Philippe.Wautelet@idris.fr>
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module mod_erreur
  contains
!*******************************************************************
REAL(kind=8) FUNCTION erreur ()
  !*****************************************************************

  USE mod_poisson

  IMPLICIT NONE

  integer :: i,j,k

  !Calcul de l'erreur
  !erreur = MAXVAL (ABS(u(sx:ex,sy:ey,sz:ez)-u_nouveau(sx:ex,sy:ey,sz:ez)))

  erreur = -1.d0
#if 0
  !Les tableaux u et u_nouveau ne sont pas parcouru de maniere
  !contigue => les acces memoires sont tres lents car le
  !cache est tres mal utilise
  do i = sx,ex
    do j = sy,ey
      do k = sz,ez
        if (ABS(u(i,j,k)-u_nouveau(i,j,k)) > erreur) then
          erreur = ABS(u(i,j,k)-u_nouveau(i,j,k))
        end if
      end do
    end do
  end do
#else
  !Les tableaux u et u_nouveau sont parcouru de maniere
  !contigue => les acces memoire utilisent au mieux les
  !caches memoire et les possibilites de prefetching
  do k = sz,ez
    do j = sy,ey
      do i = sx,ex
        if (ABS(u(i,j,k)-u_nouveau(i,j,k)) > erreur) then
          erreur = ABS(u(i,j,k)-u_nouveau(i,j,k))
        end if
      end do
    end do
  end do
#endif
END FUNCTION erreur
end module mod_erreur