!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! initialisation.f90  --- Initialisation des valeurs.
!!                 Calcul de la solution exacte u_exact et du second membre f.
!!
!! Auteur          : Isabelle DUPAYS (CNRS/IDRIS - France)
!!                   <Isabelle.Dupays@idris.fr>
!! Cree le         : Fri Nov 15 10:32:45 1996
!! Modifie par     : Philippe WAUTELET (CNRS/IDRIS - France)
!!                   <Philippe.Wautelet@idris.fr>
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module mod_init
  contains
!*******************************************************************
SUBROUTINE initialisation()
  !*****************************************************************
  USE mod_poisson

  IMPLICIT NONE

  !--Declaration des variables
  INTEGER      :: i,j,k
  real(kind=8) :: x,y,z
  real(kind=8) :: xm1,ym1,zm1
  real(kind=8) :: h3
  !*****************************************************************

  !Initialisation des matrices
  u(:,:,:)         = 0.
  u_nouveau(:,:,:) = 0.
  f(:,:,:)         = 0.
  u_exact(:,:,:)   = 0.

  !Pas
  h = 1./real(ntx+1)
  h3 = h*h*h

  !Borne superieure suivant x
  ex = UBOUND(u,DIM=1)-1

  !Borne superieure suivant y
  ey = UBOUND(u,DIM=2)-1

  !Borne superieure suivant z
  ez = UBOUND(u,DIM=3)-1

  !Initialisation du second membre et calcul de la solution exacte
  !Version optimisee
  DO k=sz,ez
    z = k*h
    zm1 = z - 1d0
    DO j=sy,ey
      y = j*h
      ym1 = y - 1d0
      DO i=sx,ex
        x = i*h
        xm1 = x - 1d0
        f(i,j,k) = 2*y*z*ym1*zm1+2*x*z*xm1*zm1+2*x*y*xm1*ym1
        u_exact(i,j,k) = x*y*z*xm1*ym1*zm1
      END DO
    END DO
  END DO

END SUBROUTINE initialisation
end module mod_init
