program recouv
  use mpi
  implicit none

  integer                               :: iter
  integer                               :: rang,nb_procs,code
  integer                               :: rang_precedent,rang_suivant
  integer,dimension(4)                  :: req
  integer,parameter                     :: etiquette=100
  integer,parameter                     :: niter=100, nx=160,ny=25000
  real(kind=8)                          :: ts,te
  real(kind=8),dimension(0:ny+1,0:nx+1) :: u,u_new

  call MPI_INIT(code)
  call MPI_COMM_RANK( MPI_COMM_WORLD, rang, code)
  call MPI_COMM_SIZE( MPI_COMM_WORLD, nb_procs, code)

  rang_precedent=mod(rang-1+nb_procs,nb_procs)
  rang_suivant=mod(rang+1,nb_procs)

  u_new(0,:)    = 0.0d0
  u_new(ny+1,:) = 1.0d0
  u_new(1:ny,:) = 0.0d0

  ts=MPI_WTIME()

  do iter=1,niter
    u(:,1:nx) = u_new(:,1:nx)

    call MPI_IRECV(u(:,nx+1),ny+2,MPI_DOUBLE_PRECISION,rang_suivant,etiquette,&
                   MPI_COMM_WORLD,req(1),code)
    call MPI_IRECV(u(:,0),ny+2,MPI_DOUBLE_PRECISION,rang_precedent,etiquette,&
                   MPI_COMM_WORLD,req(2),code)

    call MPI_ISEND(u(:,1),ny+2,MPI_DOUBLE_PRECISION,rang_precedent,etiquette,&
                   MPI_COMM_WORLD,req(3),code)
    call MPI_ISEND(u(:,nx),ny+2,MPI_DOUBLE_PRECISION,rang_suivant,etiquette,&
                   MPI_COMM_WORLD,req(4),code)

    call calcul(2,nx-1,1,ny)

    call MPI_WAITALL(4,req(:),MPI_STATUSES_IGNORE,code)

    call calcul(1,1,1,ny)
    call calcul(nx,nx,1,ny)
  end do

  te=MPI_WTIME()

  if(rang==0) print *,'Temps : ',te-ts,'s (',nx,'x',ny,')'

  call MPI_FINALIZE(code)

contains
  subroutine calcul(sx,ex,sy,ey)
    integer,intent(in)::sx,ex,sy,ey

    integer:: i,j

    do j=sx,ex
      do i=sy,ey
        u_new(i,j)=0.25*(u(i+1,j)+u(i-1,j)+u(i,j-1)+u(i,j+1))
      end do
    end do
  end subroutine calcul
end program recouv
