!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -*- Mode: F90 -*- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! calcul.f90  --- Calcul de la solution u a l'iteration n+1.
!!
!! Auteur          : Isabelle DUPAYS (CNRS/IDRIS - France)
!!                   <Isabelle.Dupays@idris.fr>
!! Cree le         : Fri Nov 15 10:32:45 1996
!! Modifie par     : Philippe WAUTELET (CNRS/IDRIS - France)
!!                   <Philippe.Wautelet@idris.fr>
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module mod_calcul
contains

!*******************************************************************
SUBROUTINE calcul()

  !*******************************************************************
  USE mod_poisson

  IMPLICIT NONE

  !Compteur
  INTEGER                               :: i,j,k
  !Pas
  REAL(kind=8)                          :: h2
  REAL(kind=8),parameter                :: onesixth = 1.d0/6.d0

  REAL(kind=8)                          :: t1,t2,tlat
  INTEGER(kind=8)                       :: flop
  REAL(kind=8)                          :: mflops,totaltime
  REAL(kind=8)                          :: minflops,maxflops,totflops

  !*******************************************************************

  h2 = h*h

  t1=mpi_wtime()

  !$OMP PARALLEL DO SCHEDULE(STATIC)
  DO k=sz, ez
    DO j=sy, ey
      DO i=sx, ex
        u_nouveau(i,j,k) = onesixth * (u(i-1,j,k)+u(i+1,j,k)+u(i,j-1,k)+u(i,j+1,k) &
                         +u(i,j,k-1)+u(i,j,k+1) - h2 * f(i,j,k))
      END DO
    END DO
  END DO
  !$OMP END PARALLEL DO

  t2=mpi_wtime()

  if (mod(it,freq_affichage)==0) then
    flop = (ez-sz+1)*(ey-sy+1)*(ex-sx+1)*8
    totaltime=t2-t1
    mflops=flop/(totaltime*1000000)

    call MPI_REDUCE(mflops,minflops,1,MPI_DOUBLE_PRECISION,MPI_MIN,0,MPI_COMM_WORLD,code)
    call MPI_REDUCE(mflops,maxflops,1,MPI_DOUBLE_PRECISION,MPI_MAX,0,MPI_COMM_WORLD,code)
    call MPI_REDUCE(mflops,totflops,1,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,code)

    if (rang==0) then
      print *,"It=",it," Time (on 0)=",totaltime," MFLOPS: min=",minflops," max=",maxflops," sum=",totflops
    end if
  end if
END SUBROUTINE calcul
end module mod_calcul
